
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Globals import DTMLFile

from Products.ZCTextIndex import ZCTextIndex
from Products.ZCTextIndex.PipelineFactory import PipelineElementFactory

from zope.interface import implements
from interfaces import IUnicodeLexicon

from UnicodeSplitter import UnicodeWordSplitter
from UnicodeSplitter import UnicodeHTMLWordSplitter
from UnicodeSplitter import UnicodeCaseNormalizer
from UnicodeSplitter import UnicodeSingleCharRemover
from UnicodeSplitter import UnicodeStopWordRemover
from UnicodeSplitter import UnicodeStopWordAndSingleCharRemover

element_factory = PipelineElementFactory()

element_factory.registerFactory('Word Splitter',
                                'Whitespace splitter',
                                UnicodeWordSplitter)

element_factory.registerFactory('Word Splitter',
                                'HTML aware splitter',
                                UnicodeHTMLWordSplitter)

element_factory.registerFactory('Case Normalizer',
                                'Case normalizer',
                                UnicodeCaseNormalizer)

element_factory.registerFactory('Stop Words',
                                'Don\'t remove stop words',
                                None)

element_factory.registerFactory('Stop Words',
                                'Remove single char words',
                                UnicodeSingleCharRemover)

element_factory.registerFactory('Stop Words',
                                'Remove stop words',
                                UnicodeStopWordRemover)

element_factory.registerFactory('Stop Words',
                                'Remove stop words and single char words',
                                UnicodeStopWordAndSingleCharRemover)


def getElementGroups(self):
    return element_factory.getFactoryGroups()

def getElementNames(self, group):
    return element_factory.getFactoryNames(group)


class UnicodeLexicon(ZCTextIndex.PLexicon):
    """Unicode Lexicon for ZCTextIndex"""

    implements(IUnicodeLexicon)

    meta_type = 'ZCTextIndex Unicode Lexicon'

    security = ClassSecurityInfo()

InitializeClass(UnicodeLexicon)


manage_addLexiconForm = DTMLFile('www/addLexicon', globals())

def manage_addLexicon(self, id, title='', elements=[], REQUEST=None):
    """Add ZCTextIndex Unicode Lexicon"""

    pipeline = []
    for el_record in elements:
        if not hasattr(el_record, 'name'):
            continue # Skip over records that only specify element group
        element = element_factory.instantiate(el_record.group, el_record.name)
        if element is not None:
            if el_record.group == 'Word Splitter':
                # I don't like hardcoding this, but its a simple solution
                # to get the splitter element first in the pipeline
                pipeline.insert(0, element)
            else:
                pipeline.append(element)

    lexicon = UnicodeLexicon(id, title, *pipeline)
    self._setObject(id, lexicon)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)

