# -*- coding: utf-8 -*-

#
# Tests for lexicon, splitters, etc.
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.UnicodeLexicon.tests import UnicodeLexiconTestCase
from Products.ZCTextIndex.Lexicon import Splitter
from Products.ZCTextIndex.Lexicon import CaseNormalizer
from Products.ZCTextIndex.Lexicon import StopWordRemover
from Products.UnicodeLexicon.UnicodeSplitter import UnicodeWordSplitter
from Products.UnicodeLexicon.UnicodeSplitter import UnicodeHTMLWordSplitter
from Products.UnicodeLexicon.UnicodeSplitter import UnicodeCaseNormalizer
from Products.UnicodeLexicon.UnicodeSplitter import UnicodeSingleCharRemover
from Products.UnicodeLexicon.UnicodeSplitter import UnicodeStopWordRemover
from Products.UnicodeLexicon.UnicodeSplitter import UnicodeStopWordAndSingleCharRemover

import locale


class TestLexicon(UnicodeLexiconTestCase.UnicodeLexiconTestCase):

    def afterSetUp(self):
        self._locale = locale.setlocale(locale.LC_ALL)

    def afterClear(self):
        locale.setlocale(locale.LC_ALL, self._locale)

    def testLower(self):
        locale.setlocale(locale.LC_ALL, 'de_DE.ISO8859-15')
        self.assertEqual('KSSEN'.lower(), 'kssen')

    def testLatin1CaseNormalizer(self):
        locale.setlocale(locale.LC_ALL, 'de_DE.ISO8859-15')
        normalize = CaseNormalizer().process
        self.assertEqual(normalize(['dland']), ['dland'])

    def testLatin1WordSplitter(self):
        locale.setlocale(locale.LC_ALL, 'de_DE.ISO8859-15')
        s = 'Schne Mdchen mu man kssen'
        split = Splitter().process
        self.assertEqual(split([s]), ['Schne', 'Mdchen', 'mu', 'man', 'kssen'])

    def testUnicodeLower(self):
        self.assertEqual(unicode('KÜSSEN', 'utf-8').lower(), u'küssen')

    def testUnicodeCaseNormalizer(self):
        normalize = UnicodeCaseNormalizer().process
        self.assertEqual(normalize(['Ödland']), [u'ödland'])

    def testUnicodeWordSplitter(self):
        s = 'Schöne Mädchen muß man küssen'
        split = UnicodeWordSplitter().process
        self.assertEqual(split([s]), [u'Schöne', u'Mädchen', u'muß', u'man', u'küssen'])

    def testUnicodeHTMLWordSplitter(self):
        s = '&#x26;<title>Schöne</title> Mädchen muß man&nbsp;küssen'
        split = UnicodeHTMLWordSplitter().process
        self.assertEqual(split([s]), [u'Schöne', u'Mädchen', u'muß', u'man', u'küssen'])

    def testLatin1StopWordRemover(self):
        locale.setlocale(locale.LC_ALL, 'de_DE.ISO8859-15')
        remover = StopWordRemover().process
        self.assertEqual(remover(['but', 'M?dchen', 'by', 'and']), ['M?dchen'])

    def testUnicodeStopWordRemover(self):
        remover = UnicodeStopWordRemover().process
        self.assertEqual(remover(['but', 'Mädchen', 'by', 'and']), [u'Mädchen'])

    def testUnicodeSingleCharWords(self):
        words = [u'\u03a4', u'\u011f', u'\u9ad8', unicode('Ä', 'utf-8')]
        for w in words:
            self.assertEqual(len(w), 1)

    def testUnicodeSingleCharRemover(self):
        remover = UnicodeSingleCharRemover().process
        self.assertEqual(remover(['\xce\xa4\xce\xb6', 'Mädchen', 'Y', '\xc4\x9f']),
                                 [u'\u03a4\u03b6', u'Mädchen'])

    def testUnicodeStopWordAndSingleCharRemover(self):
        remover = UnicodeStopWordAndSingleCharRemover().process
        self.assertEqual(remover(['\xce\xa4\xce\xb6', 'Mädchen', 'Y', '\xc4\x9f', 'by']),
                                 [u'\u03a4\u03b6', u'Mädchen'])


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestLexicon))
    return suite

if __name__ == '__main__':
    framework()

